﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace howto_overlay_with_mask
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Bitmap OverlayImage = null;
        private Bitmap ScaledOverlayImage = null;

        private Bitmap MainImage = null;
        private Bitmap ScaledMainImage = null;

        private Bitmap result_image = null;
        private Bitmap ScaledResultImage = null;

        private Bitmap MaskImage = null;
        private Bitmap ScaledMaskImage = null;

        private float ImageScale = 1;

        // Let the user select an image file.
        private void mnuFileOpenOverlay_Click(object sender, EventArgs e)
        {
            ofdImage.Title = "Open Overlay Image";
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    OverlayImage = LoadBitmapUnlocked(ofdImage.FileName);
                    ShowScaledImages();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileOpenMainImage_Click(object sender, EventArgs e)
        {
            ofdImage.Title = "Open Main Image";
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    MainImage = LoadBitmapUnlocked(ofdImage.FileName);
                    ShowScaledImages();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileOpenMaskImage_Click(object sender, EventArgs e)
        {
            ofdImage.Title = "Open Mask Image";
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    MaskImage = LoadBitmapUnlocked(ofdImage.FileName);
                    ShowScaledImages();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        // Save the selected area.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (result_image == null) return;

            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                // Save the new Bitmap.
                SaveImage(result_image, sfdImage.FileName);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Scale a bitmap.
        private Bitmap ScaleBitmap(Bitmap bm, float scale)
        {
            int scaled_width = (int)(bm.Width * scale);
            int scaled_height = (int)(bm.Height * scale);
            Bitmap result = new Bitmap(scaled_width, scaled_height);
            using (Graphics gr = Graphics.FromImage(result))
            {
                Rectangle dest_rect = new Rectangle(
                    0, 0, scaled_width, scaled_height);
                Rectangle src_rect = new Rectangle(
                    0, 0, bm.Width, bm.Height);
                gr.DrawImage(bm, dest_rect, src_rect, GraphicsUnit.Pixel);
            }
            return result;
        }

        private void mnuScale_Click(object sender, EventArgs e)
        {
            // Get the scale factor.
            ToolStripMenuItem menu_item = sender as ToolStripMenuItem;
            string scale_text = menu_item.Text.Replace("&", "").Replace("%", "");
            ImageScale = float.Parse(scale_text) / 100f;
            ShowScaledImages();

            // Display the new scale.
            mnuScale.Text = "Scale (" + menu_item.Text.Replace("&", "") + ")";

            // Check the selected menu item.
            foreach (ToolStripMenuItem item in mnuScale.DropDownItems)
            {
                item.Checked = (item == menu_item);
            }
        }

        private void ShowScaledImages()
        {
            if (OverlayImage != null)
            {
                ScaledOverlayImage = ScaleBitmap(OverlayImage, ImageScale);
                picOverlay.Image = ScaledOverlayImage;
                picOverlay.Visible = true;
            }

            if (MainImage != null)
            {
                ScaledMainImage = ScaleBitmap(MainImage, ImageScale);
                picMain.Image = ScaledMainImage;
                picMain.Visible = true;
            }

            if (MaskImage != null)
            {
                ScaledMaskImage = ScaleBitmap(MaskImage, ImageScale);
                picMask.Image = ScaledMaskImage;
                picMask.Visible = true;
            }

            // Combine the images.
            CombineImages();

            // Enable the Save As menu item.
            mnuFileSaveAs.Enabled = (
                (OverlayImage != null) &&
                (MainImage != null));
        }

        private void CombineImages()
        {
            result_image = OverlayWithMask(MainImage, OverlayImage, MaskImage);
            if (result_image == null) return;

            ScaledResultImage = ScaleBitmap(result_image, ImageScale);
            picResult.Image = ScaledResultImage;
            picResult.Visible = true;
        }

        private Bitmap OverlayWithMask(Bitmap main_image,
            Bitmap overlay_image, Bitmap mask_image)
        {
            if (main_image == null || overlay_image == null || mask_image == null)
                return null;

            result_image = new Bitmap(main_image);
            Bitmap32 bm32_main = new Bitmap32(main_image);
            Bitmap32 bm32_overlay = new Bitmap32(overlay_image);
            Bitmap32 bm32_mask = new Bitmap32(mask_image);
            Bitmap32 bm32_result = new Bitmap32(result_image);
            bm32_main.LockBitmap();
            bm32_overlay.LockBitmap();
            bm32_mask.LockBitmap();
            bm32_result.LockBitmap();
            for (int y = 0; y < bm32_main.Height; y++)
            {
                for (int x = 0; x < bm32_main.Width; x++)
                {
                    byte r, g, b, a;
                    bm32_mask.GetPixel(x, y, out r, out g, out b, out a);
                    if (r == 0 && g == 0 && b == 0)
                    {
                        bm32_overlay.GetPixel(x, y, out r, out g, out b, out a);
                        bm32_result.SetPixel(x, y, r, g, b, a);
                    }
                }
            }
            bm32_main.UnlockBitmap();
            bm32_overlay.UnlockBitmap();
            bm32_mask.UnlockBitmap();
            bm32_result.UnlockBitmap();

            return result_image;
        }
    }
}
